package NoRFC::Const;
use strict;
use utf8;

=encoding utf8

=head1 NAME

NoRFC::Const - NoRFC constants

=head1 VERSION

Version 1.00

=head1 SYNOPSIS

    use NoRFC::Const;

=head1 DESCRIPTION

This module contains constants definitions

=head2 TAGS

=over 8

=item B<:GENERAL>

Exports common constants

=item B<:MAPS>

Exports constants of maps

=item B<:NUMBER>

Exports only constants for a document number parsing and generation

=item B<:SERVER>

Exports only constants for a preforked server

=back

=head1 HISTORY

See C<Changes> file

=head1 TO DO

See C<TODO> file

=head1 SEE ALSO

L<NoRFC>

=head1 AUTHOR

Serż Minus (Sergey Lepenkov) L<http://www.serzik.com> E<lt>minus@serzik.comE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2023 D&D Corporation. All Rights Reserved

=head1 LICENSE

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

See C<LICENSE> file

=cut

use vars qw/$VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS/;
$VERSION = '1.00';

use base qw/Exporter/;

use Try::Tiny;

use constant {
    PROJECTNAME         => "NoRFC",
    PROJECTNAMEL        => "norfc",
    DATADIR             => "norfc",
    YFILE               => "norfc.yml",
    INDEXFILE           => "index.yml",
    DBFILE              => "norfc.db",
    DBDIR               => "db",
    SNAPSHOTDIR         => "snapshots", # For export
    RESTOREDIR          => "restores", # For import

    # System constants
    IS_TTY              => (-t STDOUT) ? 1 : 0,
    IS_ROOT             => ($> == 0) ? 1 : 0,
    SCREENWIDTH_DEFAULT => 80,
    QUIET_MODE          => 1,
    TARBIN              => 'tar',

    # Date formats
    DATE_FORMAT         => "%YYYY-%MM-%DD",
    DATETIME_FORMAT     => "%YYYY-%MM-%DD %hh:%mm:%ss",
    DATETIME_GMT_FORMAT => "%YYYY-%MM-%DD %hh:%mm:%ss %G",

    # NoRFC numbers
    NORFCNUM_CANONICAL  => "%s-%s%d%03d%d%d", # <TYPE>-<STATUS><YEAR><YDAY><NN><M11R>
    NORFCNUM_DIGITS_V1  => "%d%03d%02d", # <YEAR><YDAY><NN>
    NORFCNUM_DIGITS_V2  => "%d%03d%d", # <YEAR><YDAY><NN>
    NORFCNUM_DOTS       => "%s%s%d.%03d.%d.%d", # [[<TYPE>-]<STATUS>.]<YEAR>.<YDAY>.<NN>.<M11R>
    NORFCNUM_SHORT      => "%s%s%d%03d%d%d", # [[<TYPE>-]<STATUS>]<YEAR><YDAY><NN><M11R>
    NORFCNUM_REGEXP_V1  => qr/^\s*([Dd][Rr][Aa][Ff][Tt]|RFC|[Nn][Oo]RFC)\-([RPXZ])([0-9]{2})([0-9]{3})([0-9]{2})([0-9]{1})\s*$/,
    NORFCNUM_REGEXP_V2  => qr/^\s*(?>([Dd][Rr][Aa][Ff][Tt]|[Nn][Oo][Rr][Ff][Cc])(?>[- ]+))?([RPXZ])?(?>[.]?)([0-9]{4})(?>[.]?)([0-9]{3})(?>[.]?)([0-9]{1,6})(?>[.]?)([0-9]{1})\s*$/,
    NORFCNUM_REGEXP_V2S => qr/\s*(?>([Dd][Rr][Aa][Ff][Tt]|[Nn][Oo][Rr][Ff][Cc])(?>[- ]+))?([RPXZ])?(?>[.]?)([0-9]{4})(?>[.]?)([0-9]{3})(?>[.]?)([0-9]{1,6})(?>[.]?)([0-9]{1})\s*/,
    NORFCNUM_NN_MAX     => 999999,

    # Defaults
    DEFAULT_CATEGORY    => "Unknown",
    DEFAULT_TITLE       => "No title",
    DEFAULT_CONTENT     => "",
    DEFAULT_AUTHOR      => "Anonymous",

    # App
    PAGE_WIDTH          => 80, # For doc wrapping
    MAX_ROWS            => 1000, # Records per page

    # Server
    SERVER_URL          => 'http://localhost:8732',
    SERVER_LISTEN_ADDR  => "*",
    SERVER_LISTEN_PORT  => 8732,
    UPGRADE_TIMEOUT     => 30,

    # UID/GID for daemon
    USERNAME            => 'norfc',
    GROUPNAME           => 'norfc',

    # MAPS
    STATUSES_MAP        => {
        R => "Regular",
        P => "Private",
        X => "Extra",
        Z => "Secret",
    },
    NORFC_CATEGORIES_MAP=> [
        "Informational",
        "Experimental",
        "Standards Track",
        "Best Current Practice",
        "Historic",
        "Unknown",
    ],
    EXTRA_CATEGORIES_MAP=> [
        "Application",
        "Library",
        "Extension",
        "Appendix",
        "Attachment",
        "Configuration",
        "Instruction",
        "User manual",
        "Etcetera",
        #"Script",
        #"Module",
        #"WEB Site",
        #"Server",
        #"Service",
    ],

};

my $myscreenw = undef;
*SCREENWIDTH = sub {
    return $myscreenw if defined $myscreenw;
    if (IS_TTY) {
        try {
            require Term::ReadKey;
            my $w = (Term::ReadKey::GetTerminalSize())[0];
            $myscreenw = $w < SCREENWIDTH_DEFAULT ? SCREENWIDTH_DEFAULT : $w;
        } catch {
            $myscreenw = SCREENWIDTH_DEFAULT;
        };
    } else {
        $myscreenw = SCREENWIDTH_DEFAULT;
    }
    return $myscreenw;
};

# Named groups of exports
%EXPORT_TAGS = (
    'GENERAL' => [qw/
        IS_TTY IS_ROOT
        QUIET_MODE
        PROJECTNAME PROJECTNAMEL
        DATADIR YFILE INDEXFILE DBFILE DBDIR SNAPSHOTDIR RESTOREDIR
        TARBIN
        SCREENWIDTH_DEFAULT SCREENWIDTH
        DATE_FORMAT DATETIME_FORMAT DATETIME_GMT_FORMAT
        PAGE_WIDTH MAX_ROWS
        DEFAULT_CATEGORY DEFAULT_TITLE DEFAULT_CONTENT DEFAULT_AUTHOR
    /],
    'NUMBER' => [qw/
        NORFCNUM_CANONICAL NORFCNUM_DIGITS_V1 NORFCNUM_DIGITS_V2 NORFCNUM_DOTS NORFCNUM_SHORT
        NORFCNUM_REGEXP_V1 NORFCNUM_REGEXP_V2 NORFCNUM_REGEXP_V2S
        NORFCNUM_NN_MAX
    /],
    'MAPS' => [qw/
        STATUSES_MAP NORFC_CATEGORIES_MAP EXTRA_CATEGORIES_MAP
    /],
    'SERVER' => [qw/
        SERVER_URL SERVER_LISTEN_ADDR SERVER_LISTEN_PORT UPGRADE_TIMEOUT
        USERNAME GROUPNAME
    /],
);

# Items to export into callers namespace by default
# (move infrequently used names to @EXPORT_OK below)
@EXPORT = (
        @{$EXPORT_TAGS{GENERAL}},
    );

# Other items we are prepared to export if requested
@EXPORT_OK = (
        map {@{$_}} values %EXPORT_TAGS
    );

1;

__END__
