package NoRFC::Number;
use strict;
use utf8;

=encoding utf8

=head1 NAME

NoRFC::Number - The noRFC numbers manipulations

=head1 VERSION

Version 1.00

=head1 SYNOPSIS

    use NoRFC::Number;

=head1 DESCRIPTION

This module provides the noRFC number interface

NoRFC Document Numbering Format (V2):

    [<TYPE>{-| }][<STATUS>[.]]<YEAR>[.]<YDAY>[.]<NN>[.]<M11R>

=over 8

=item B<TYPE>

Document types

    NoRFC   Regular noRFC documents (default)
    Draft   Draft of noRFC document

=item B<STATUS>

Status letter

    R (Regular)        Regular public document (default)
    P (Private)        Private document, access limited by password
    X (Extra)          Extra protected document (PGP encrypted document)
    Z (Top secret)     Top secret document (encrypted by third-party tool)

=item B<YEAR>

The four digits of the current year (e.g., 2020)

Length: 4 digits

=item B<YDAY>

It is the day of the year, in the range 001..365 (or 001..366 in leap years)

Length: 3 digits

=item B<NN>

Sequence number in the current day, in the range 1..999999

Length: 1-6 digits

=item B<M11R>

Check digit by M11-Recursive Algorithm

Length: 1 digit

=back

B<V1 Examples:>

    Draft-R20162124
    NoRFC-R20162124
    NoRFC-R20162124

B<V2 Examples:>

    NoRFC-R2020162124
    NoRFC R2020162124
    Draft R2020162124
    R2020162124
    2020162124
    R.2020.162.12.4

=head2 new

    my $norfcnum = NoRFC::Number->new;
    my $norfcnum = NoRFC::Number->new( "string" );

Creates new object instance and parses string if it specified

=head2 as_dotted

    my $string = $norfcnum->as_dotted; # 2020.162.123456.8

Returns the noRFC number as short dotted string:

    [[<TYPE>-]<STATUS>.]<YEAR>.<YDAY>.<NN>.<M11R>

=head2 as_hash

    my %norfcnum = $norfcnum->as_hash;

Returns hash-structure, eg.:

    # Draft-R21050019
    (
      'ver'     => 1,
      'type'    => 'Draft',
      'status'  => 'R',
      'year'    => 2021,
      'yday'    => 50,
      'nn'      => 1,
      'm11r'    => 9,
    )


=head2 as_short

    my $string = $norfcnum->as_short; # 20201621234568

Returns the noRFC number as short string:

    [[<TYPE>-]<STATUS>]<YEAR><YDAY><NN><M11R>

=head2 as_string

    my $string = $norfcnum->as_string; # NoRFC-R20201621234568

Returns the noRFC number as string

=head2 clone

    my $copy = $norfcnum->clone;

Returns a copy of the $norfcnum object

=head2 is_valid

    $norfcnum->is_valid or die "The noRFC number is invalid";

Performs validation of the noRFC number and returns booly status

=head2 m11r

    my $m11r = $norfcnum->m11r;

Returns the m11r value of the noRFC number

=head2 make

    $norfcnum = $norfcnum->make(
      'type'    => 'Draft',
      'status'  => 'R',
      'year'    => 2021,
      'yday'    => 50,
      'nn'      => 1,
    );

Make new object by hash data

=head2 nn

    my $nn = $norfcnum->nn;

Returns the nn value of the noRFC number

=head2 parse

    $norfcnum = $norfcnum->parse( "Draft-R21050019" );
    $norfcnum = $norfcnum->parse( "NoRFC-R2020162124" );

Parse the noRFC document number and returns its object

=head2 status

    my $status = $norfcnum->status;

Returns the status of the noRFC number

=head2 type

    my $type = $norfcnum->type;

Returns the type of the noRFC number

=head2 ver

    my $version = $norfcnum->ver;

Returns the version of the noRFC number

=head2 yday

    my $yday = $norfcnum->yday;

Returns the yday of the noRFC number

=head2 year

    my $year = $norfcnum->year;

Returns the year of the noRFC number

=head1 SEE ALSO

L<NoRFC>

=head1 AUTHOR

Serż Minus (Sergey Lepenkov) L<http://www.serzik.com> E<lt>minus@serzik.comE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2023 D&D Corporation. All Rights Reserved

=head1 LICENSE

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

See C<LICENSE> file

=cut

use vars qw/ $VERSION /;
$VERSION = '1.00';

use NoRFC::Const qw/ :NUMBER /;
use NoRFC::Util qw/ m11r_checkdigit /;

sub new {
    my $class = shift;
    my $string = shift;

    my $self = bless {
            orig        => $string,
            ver         => 0,
            type        => "NoRFC",
            status      => "R",
            year        => 0,
            yday        => 0,
            nn          => 0,
            m11r        => 0,
            m11r_got    => 0,
            is_valid    => 0,
        }, $class;

    return $self->parse($string) if defined $string;
    return $self;
}

sub ver {
    my $self = shift;
    return $self->{ver} || 0;
}
sub type {
    my $self = shift;
    return $self->{type} || "NoRFC";
}
sub status {
    my $self = shift;
    return $self->{status} || "R";
}
sub year {
    my $self = shift;
    return $self->{year} || 0;
}
sub yday {
    my $self = shift;
    return $self->{yday} || 0;
}
sub nn {
    my $self = shift;
    return $self->{nn} || 0;
}
sub m11r {
    my $self = shift;
    return $self->{m11r} || 0;
}
sub is_valid {
    my $self = shift;

    my $ver = $self->ver;
    my $year = $self->year;

    # M11R validation
    my $got;
    if ($ver == 1) {
        $got = m11r_checkdigit(sprintf(NORFCNUM_DIGITS_V1, $year - 2000, $self->yday, $self->nn));
    } elsif ($ver == 2) {
        $got = m11r_checkdigit(sprintf(NORFCNUM_DIGITS_V2, $year, $self->yday, $self->nn));
    }
    $self->{m11r_got} = $got;
    $self->{is_valid} = defined($got)
        ? ($self->m11r == $got) ? 1 : 0
        : 0;

    return $self->{is_valid};
}

sub parse {
    my $self = shift;
    my $number = shift;
    return $self unless defined $number;
    $self->{orig} = $number;

    # V1
    if ($number =~ NORFCNUM_REGEXP_V1) {
        my ($t, $s, $y, $d, $n, $m) = ($1, $2, $3, $4, $5, $6);

        # Version
        $self->{ver} = "1";

        # Type
        if (!defined($t)) {
            $self->{type} = 'NoRFC';
        } elsif ($t =~ /draft/i) {
            $self->{type} = 'Draft';
        } else {
            $self->{type} = 'NoRFC';
        }

        # Status
        $self->{status} = defined($s) ? uc($s) : "R";

        # Year, yday
        my $y3   = defined($y) ? $y*1 : 0;
        $self->{year} = 2000 + $y3;
        $self->{yday} = defined($d) ? $d*1 : 0;

        # NN
        $self->{nn} = defined($n) ? $n*1 : 0;

        # M11R
        $self->{m11r} = defined($m) ? $m*1 : 0;

    # V2
    } elsif ($number =~ NORFCNUM_REGEXP_V2) {
        my ($t, $s, $y, $d, $n, $m) = ($1, $2, $3, $4, $5, $6);

        # Version
        $self->{ver} = "2";

        # Type
        if (!defined($t)) {
            $self->{type} = 'NoRFC';
        } elsif ($t =~ /draft/i) {
            $self->{type} = 'Draft';
        } else {
            $self->{type} = 'NoRFC';
        }

        # Status
        $self->{status} = defined($s) ? uc($s) : "R";

        # Year, yday
        $self->{year} = defined($y) ? $y*1 : 0;
        $self->{yday} = defined($d) ? $d*1 : 0;

        # NN
        $self->{nn} = defined($n) ? $n*1 : 0;

        # M11R
        $self->{m11r} = defined($m) ? $m*1 : 0;
    }

    # $ret{dir} = File::Spec->catdir($ret{y3}, sprintf("%d%03d", $ret{y3}, $ret{yday})); # <Y3>/<Y3><YDAY>

    return $self;
}
sub clone {
    my $self = shift;
    my %other = %$self;
    bless {%other}, ref $self;
}
sub as_hash {
    my $self = shift;
    return (
      'ver'     => $self->ver,
      'type'    => $self->type,
      'status'  => $self->status,
      'year'    => $self->year,
      'yday'    => $self->yday,
      'nn'      => $self->nn,
      'm11r'    => $self->m11r,
    );
}
sub as_string {
    my $self = shift;
    return sprintf(NORFCNUM_CANONICAL,
        $self->type, $self->status, $self->year, $self->yday, $self->nn,
        $self->m11r);
    #m11r_checkdigit(sprintf(NORFCNUM_DIGITS_V2, $year, $yday, $cnt)));
}
sub as_short {
    my $self = shift;
    my $type = $self->type // '';
       $type = "" if $type eq 'NoRFC';
       $type .= "-" if length $type;
    my $status = $self->status // '';
       $status = '' if $status eq 'R';
    return sprintf(NORFCNUM_SHORT, # "%s%s%d%03d%d%d"
        $type, $status, $self->year, $self->yday, $self->nn,
        $self->m11r);
}
sub as_dotted {
    my $self = shift;
    my $type = $self->type // '';
       $type = "" if $type eq 'NoRFC';
       $type .= "-" if length $type;
    my $status = $self->status // '';
       $status = '' if $status eq 'R';
       $status .= "." if length $status;
    return sprintf(NORFCNUM_DOTS, # "%s%s%d.%03d.%d.%d", # [[<TYPE>-]<STATUS>.]<YEAR>.<YDAY>.<NN>.<M11R>
        $type, $status, $self->year, $self->yday, $self->nn,
        $self->m11r);
}
sub make {
    my $self = shift;
    my %d = @_;

    $self->{ver} = 2;
    $self->{type} = $d{type} && $d{type} =~ /draft/i ? "Draft" : "NoRFC";
    $self->{status} = ($d{status} && $d{status} =~ /^[RPXZ]$/i) ? uc($d{status}) : "R";
    $self->{year} = $d{year} || 0;
    $self->{yday} = $d{yday} || 0;
    $self->{nn} = $d{nn} || 0;
    $self->{m11r} = $d{m11r} // m11r_checkdigit(sprintf(NORFCNUM_DIGITS_V2, $self->{year}, $self->{yday}, $self->{nn}));

    return $self;
}


1;

__END__
