package NoRFC::Server::Alpha;
use utf8;

=encoding utf8

=head1 NAME

NoRFC::Server::Alpha - The primary (alpha) mojolicious noRFC controller

=head1 VERSION

Version 1.01

=head1 SYNOPSIS

    use NoRFC::Server::Alpha;

=head1 DESCRIPTION

The primary (alpha) mojolicious noRFC controller

=head2 publish

Publish the noRFC document

=head2 env

Gets environments values

=head2 gendocnum

Generates a new noRFC document number

=head2 info

Gets general information

=head2 norfc

Returns the noRFC document

=head2 reladd

Add new relation of the noRFC document number

=head2 reldel

Delete relations of the noRFC document number

=head2 relget

Get relations of the noRFC document number

=head2 remove

Remove the noRFC document by document name

=head2 root

The root controller

=head2 search

Performs searching the noRFC documents

=head1 SEE ALSO

L<Mojolicious>, L<YAML::XS>

=head1 AUTHOR

Serż Minus (Sergey Lepenkov) L<http://www.serzik.com> E<lt>minus@serzik.comE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2023 D&D Corporation. All Rights Reserved

=head1 LICENSE

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

See C<LICENSE> file

=cut

use vars qw/ $VERSION /;
$VERSION = "1.01";

use Mojo::Base 'Mojolicious::Controller';

use Mojo::File qw/path/;
use Mojo::JSON qw/from_json/;
use Mojo::Util qw/b64_decode/;

use Encode qw/encode decode/;
use Try::Tiny;
use YAML::XS qw/DumpFile LoadFile/;

use CTK::Util qw/sharedir sharedstatedir trim dformat/;
use CTK::ConfGenUtil;
use CTK::TFVals qw/ :ALL /;

use NoRFC::Document;
use NoRFC::Const qw/ :GENERAL :NUMBER /;
use NoRFC::Util qw/filesize md5sum sha1sum sha256sum attr2hash hash2attr/;
use NoRFC::Number;

use constant {
        CONTENT_TYPE_JSON   => "application/json",
        CONTENT_TYPE_TXT    => "text/plain; charset=utf-8",
    };

sub root {
    my $self = shift;
    return $self->reply->static('index.html');
}
sub env {
    my $self = shift;
    $self->res->headers->content_type(CONTENT_TYPE_TXT);
    return $self->render(text => $self->dumper(\%ENV));
}
sub info {
    my $self = shift; # Mojolicious::Routes
    my $ctk = $self->app->ctk();
    my $store = $ctk->store;
    my $rfcnum = $self->param("rfcnum");

    #$self->app->log->debug("Catched!");
    if ($rfcnum) {
        my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);

        # Check number
        return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;

        # Store check
        return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

        # Get data from DB
        my %info = $store->getinfo($norfcnum->as_string);
        return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

        # Get relations
        my @relations = $store->relget($norfcnum->as_string);
        return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;
        my (@updates, @obsoletes);
        foreach my $rel (@relations) {
            if ($rel->{name} eq 'Updates') {
                push @updates, $rel->{external};
            } elsif ($rel->{name} eq 'Obsoletes') {
                push @obsoletes, $rel->{external};
            }
        }
        $info{updates} = [sort {$a cmp $b} @updates];
        $info{obsoletes} = [sort {$a cmp $b} @obsoletes];

        # Get attributes
        my %attr = attr2hash($info{attributes});

        # File in database directory
        my $filepath = path($self->app->dbdir, $norfcnum->year,
            sprintf("%d%03d", $norfcnum->year, $norfcnum->yday),
            sprintf("%s.raw", $norfcnum->as_string)); # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw>
        if ($attr{external} && -e $filepath->to_string) {
            $info{file} = $filepath->to_string;
            $info{size} = filesize($filepath->to_string);
            $info{sha1} = sha1sum($filepath->to_string);
            $info{sha256} = sha256sum($filepath->to_string);
            $info{md5}  = md5sum($filepath->to_string);
        }

        # Result
        return $self->render(json => {
            status  => 1,
            message => $info{id} ? "Found" : "Not found",
            %info,
        });
    }

    # Check permissions (Authentication and Authorization)
    my $authorized = $self->is_authenticated && $self->is_authorized ? 1 : 0;
    $self->res->headers->header('X-Authorized' => $authorized);

    # General info
    my $cnt = $store->count || 0;
    return $self->render(json => {
            name => "info",
            status => $store->status,
            message => $store->status ? "Ok" : $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
            is_authorized => $authorized,
            dsn => $store->dsn,
            total => $cnt,
            version => $ctk->VERSION,
            categories => [($ctk->categories(1), $ctk->categories(2))],
            datadir => $ctk->datadir,
            maxstorefilesize => $self->app->maxstorefilesize,
            $ctk->debugmode() ? (
                homedir => $self->app->home(),
                documentroot => $self->app->documentroot,
                tempdir => $ctk->tempdir,
                paths => $self->app->static->paths(),
                dbdir => $self->app->dbdir,
            ) : (),
        });
}
sub gendocnum {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $store = $ctk->store;

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Files
    my $yfile = File::Spec->catfile($ctk->datadir, YFILE);

    # Date data
    my ($mday, $mon, $year, $yday) = (localtime(time))[3,4,5,7];
    $mon++; $yday++; $year += 1900;
    $year = $self->param("year") if $self->param("year");
    $yday = $self->param("yday") if $self->param("yday");

    # D-Key
    my $dkey = sprintf("%04d-%02d-%02d", $year, $mon, $mday);

    # Type
    my $type = $self->param("type") || 'NoRFC';
    return $self->render(json => {
            status => 0,
            message => sprintf("Incorrect the type value"),
        }, status => 400) if $type !~ /Draft|NoRFC/;

    # Status
    my $status = $self->param("status") || 'R';
    return $self->render(json => {
            status => 0,
            message => sprintf("Incorrect the status value"),
        }, status => 400) if $status !~ /^[RPXZ]$/;

    # Sequence number (counter)
    my $nn = $self->param("nn") // 0;
    unless ($nn) {
        my $history = {};
        if (-e $yfile and -s _) { # Load history data
            my $err = "";
            try {
                $history = LoadFile($yfile);
            } catch {
                $err = sprintf("Can't load file %s: %s", $yfile, $_ // "unknown error");
            };
            return $self->render(json => {
                    status => 0,
                    message => $err,
                }, status => 500) if $err;
        }
        $nn = ($history->{$dkey} || 0) + 1;
        return $self->render(json => {
                status => 0,
                message => "No free counter values. Try specify year or day manually",
            }, status => 500) if $nn > NORFCNUM_NN_MAX;
    }

    # Generate a new number
    my $num = "";
    my $short = "";
    my $dotted = "";
    while ($nn <= NORFCNUM_NN_MAX) {
        my $norfcnum = NoRFC::Number->new->make(type => $type, status => $status, year => $year, yday => $yday, nn => $nn);
           $num = $norfcnum->as_string;
           $short = $norfcnum->as_short;
           $dotted = $norfcnum->as_dotted;
        my %info = $store->get($num);
        unless ($store->status) {
            return $self->render(json => {
                status => 0,
                message => $store->error || sprintf("Store (%s) unknown error", $store->dsn),
            }, status => 500)
        }
        last unless $info{id}; # Not Found
    } continue {
        $nn++;
    }
    return $self->render(json => {
            status => 0,
            message => "No free counter values. Try specify year or day manually",
        }, status => 500) if $nn > NORFCNUM_NN_MAX;

    # Save history data
    if ($nn) {
        my $err = "";
        try {
            DumpFile($yfile, {$dkey => $nn});
        } catch {
            $err = sprintf("Can't write file \"%s\": %s", $yfile, $_ // "unknown error");
        };
        return $self->render(json => {
                status => 0,
                message => $err,
            }, status => 500) if $err;
    }

    # Result
    return $self->render(json => {
        message => "Ok",
        year    => $year,
        yday    => $yday,
        dkey    => $dkey,
        type    => $type,
        status  => $status,
        nn      => $nn,
        number  => $num,
        short   => $short,
        dotted  => $dotted,
    });
}
sub search {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $store = $ctk->store;
    my @found = ();

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Get data for search
    my $in_text = trim($self->param("text") || '');
    my $in_category = trim($self->param("category") || '');
    my $in_page = trim($self->param("page") || 0);
    my $in_limit = trim($self->param("limit")) || MAX_ROWS;
    my ($page, $limit, $pages, $total, $count) = (0, 0, 0, 0, 0);

    # Scope
    my $by_num = $in_text =~ NORFCNUM_REGEXP_V2 ? 1 : 0;
    if ($by_num) { # Search by number
        my $norfcnum = $ctk->is_valid($in_text, QUIET_MODE);
        if ($norfcnum) {
            my $docnum = $norfcnum->as_string;

            # Get data from DB
            my %info = $store->getinfo($docnum);
            return $self->render(json => {
                status => 0,
                message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
            }, status => 500) unless $store->status;

            # Found
            push @found, {
                n           => ++$total,
                id          => $info{id} || 0,
                num         => $info{num} || '',
                pub         => $info{pub} || 0,
                category    => $info{category} || '',
                title       => $info{title} || '',
            } if $info{id};
            if ($total) {
                $count = 1;
                $pages = 1;
            }
        }
    } else { # Search by title/category
        # Get count
        ($count) = $store->search(
            get_count_only => 1,
            category    => $in_category,
            title       => $in_text,
        );
        return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

        # Get offset, limit and page number
        $count ||= 0;
        $limit = is_int($in_limit) && $in_limit <= MAX_ROWS ? $in_limit : MAX_ROWS;
        $pages = (($count - ($count % $limit)) / $limit) + (($count % $limit) ? 1 : 0);
        $page = is_int($in_page) ? $in_page : 0;
        $page = 1 if $page > $pages;

        # Get
        my @rows = $store->search(
            offset      => $page ? ($page - 1) * $limit : 0,
            row_count   => $limit,
            category    => $in_category,
            title       => $in_text,
        );
        return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

        # Found
        foreach my $row (@rows) {
            push @found, {
                n           => ++$total,
                id          => $row->[0] || 0,
                num         => $row->[1] || '',
                pub         => $row->[2] || 0,
                category    => $row->[3] || '',
                title       => $row->[4] || '',
            };
        }
    }

    # Result
    return $self->render(json => {
            status => 1,
            message => scalar(@found) ? "Found" : "Not found",
            found => [@found],
            in => {
                text        => $in_text,
                category    => $in_category,
            },
            by_number   => $by_num ? "yes" : "no",
            pages       => $pages,
            page        => $page || 1,
            limit       => $limit,
            total       => $total,
            count       => $count,
        });
}
sub norfc {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $store = $ctk->store;
    #$self->app->log->info(" !!!! " . $rfcnum);

    # The doc or raw file required?
    my $p = $self->req->url->path // "";
    my $req_raw = ($p =~ /\.(raw|src)\/?$/) ? 1 : 0;

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;
    my $docnum = $norfcnum->as_string;

    # Check permissions
    if ($norfcnum->status eq "P") {
        # Authentication
        return unless $self->is_authenticated({ render_by_fail => 1 });

        # Authorization
        return unless $self->is_authorized({ render_by_fail => 1 });
    }

    # Check store instance
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Get document info from DB
    my %info = $store->get($docnum);
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;
    return $self->render(json => {
            status => 0,
            message => "Document not found",
        }, status => 404) unless $info{id};

    # Get attributes
    my %attr = attr2hash($info{attributes});

    # Temp file and path
    my $filerawname = sprintf("%s.tmp", $docnum);
    my $path_fileraw = path($ctk->tempdir, $filerawname);
    my $fileraw = $path_fileraw->to_string;

    # File in database directory
    my $filepath = path($self->app->dbdir, $norfcnum->year,
        sprintf("%d%03d", $norfcnum->year, $norfcnum->yday),
        sprintf("%s.raw", $norfcnum->as_string)); # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw>

    # Save content to temp file
    if ($attr{external} && -e $filepath->to_string) {
        # Copy file from db to temp file
        $filepath->copy_to($fileraw);
        my $md5 = md5sum($fileraw);
        return $self->render(json => {
            message => sprintf("File md5 checksum mismatch. Expected: %s; Got: %s", $attr{md5}, $md5),
        }, status => 500) unless !$attr{md5} or $attr{md5} eq $md5;
    } else {
        # Save content to temp file
        Encode::_utf8_off($info{content}) if defined $info{content};
        $path_fileraw->spurt($info{content} // DEFAULT_CONTENT());
        # Check file
        return $self->render(json => {
                message => "Can't save file \"$fileraw\"",
            }, status => 500) unless -e $fileraw;
    }

    # Send only raw file if required
    return $self->reply->static($filerawname) if $req_raw;

    # Get relations
    my @relations = $store->relget($norfcnum->as_string);
    return $self->render(json => {
        status => 0,
        message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
    }, status => 500) unless $store->status;
    my (@updates, @obsoletes);
    foreach my $rel (@relations) {
        if ($rel->{name} eq 'Updates') {
            push @updates, $rel->{external};
        } elsif ($rel->{name} eq 'Obsoletes') {
            push @obsoletes, $rel->{external};
        }
    }

    # Create document
    my $doc = NoRFC::Document->new(
        number      => $docnum,
        category    => $info{category} || DEFAULT_CATEGORY(),
        published   => $info{pub},
        author      => $info{author} || DEFAULT_AUTHOR(),
        title       => $info{title} || DEFAULT_TITLE(),
        columns     => $attr{columns},
        tabstop     => $attr{tabstop},
        updates     => [@updates],
        obsoletes   => [@obsoletes],
    );
    if ($attr{external} || $attr{binary} || $attr{encrypted}) {
        $doc->file($fileraw);
    } else {
        $doc->body($info{content} // DEFAULT_CONTENT());
    }

    # Output
    $self->res->headers->content_type(CONTENT_TYPE_TXT);
    return $self->render(text => $doc->as_string);
}
sub publish {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $store = $ctk->store;
    #$self->app->log->info(" !!!! " . $rfcnum);

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;
    my $docnum = $norfcnum->as_string;

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Get attributes
    my %attr = attr2hash($self->param("attributes"));

    # Upload file @fileraw@
    my $path_fileraw = path($ctk->tempdir, sprintf("%s.upl", $docnum));
    my $raw_uploaded = $self->req->upload('fileraw');
    if ($raw_uploaded) {
        my $file = $path_fileraw->to_string;
        my $size = $raw_uploaded->size;
        $raw_uploaded->move_to($file);

        # Check file
        unless (-e $file) {
            return $self->render(json => {
                message => "Can't upload file \"$file\"",
            }, status => 500);
        }

        # Check size
        if ($self->param("size") && $self->param("size") != $size) {
            $path_fileraw->remove;
            return $self->render(json => {
                message => "File size mismatch",
            }, status => 400);
        }
        #$attr{size} = $size;

        # Check md5sum
        if ($self->param("md5") && $self->param("md5") ne md5sum($file)) {
            $path_fileraw->remove;
            return $self->render(json => {
                message => "File md5 checksum mismatch",
            }, status => 400);
        }
        $attr{md5} = $self->param("md5");

        # Fix external mode
        $attr{external} = 1 if $size > $self->app->maxstorefilesize;
    } else {
        $path_fileraw->touch;
    }

    # Upload file @filesig@
    my $path_filesig = path($ctk->tempdir, sprintf("%s.upl.sig", $docnum));
    my $sig_uploaded = $self->req->upload('filesig');
    if ($sig_uploaded) {
        my $file = $path_filesig->to_string;
        my $size = $sig_uploaded->size;
        $sig_uploaded->move_to($file);

        # Check file
        unless (-e $file) {
            return $self->render(json => {
                message => "Can't upload file \"$file\"",
            }, status => 500);
        }

        # Check size
        unless ($size) {
            $path_filesig->remove;
            return $self->render(json => {
                message => "File size incorrect",
            }, status => 400);
        }
    } else {
        $path_filesig->touch;
    }

    # Store file to dbdir
    if ($attr{external}) {
        my $path_dir = path($self->app->dbdir, $norfcnum->year,
            sprintf("%d%03d", $norfcnum->year, $norfcnum->yday))->make_path; # db/<Y3>/<Y3><YDAY>
        my $path_filebin = path($path_dir, sprintf("%s.raw", $docnum)); # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw>
           $path_fileraw->move_to($path_filebin->to_string);
           my $md5 = md5sum($path_filebin->to_string);
        my $path_filemd5 = path($path_dir, sprintf("%s.raw.md5", $docnum)); # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw.md5>
           $path_filemd5->spurt(sprintf("%s  %s.raw\n", $md5, $docnum));
        $self->app->log->debug(sprintf("Publish=%s, ExternalFile=%s, Size=%s, MD5=%s",
            $docnum, $path_filebin->to_string, $attr{size} // '', $md5));
        return $self->render(json => {
            message => "File md5 checksum mismatch",
        }, status => 500) unless $self->param("md5") && $md5 eq $self->param("md5");
    }

    # Get ID of existed record
    my $id = $store->getid($docnum) || 0;
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Publish!
    my %args = (
            $id ? (id => $id) : (),
            num         => $docnum,
            pub         => $self->param("published"),
            category    => $self->param("category"),
            title       => $self->param("title"),
            author      => $self->param("author"),
            attributes  => hash2attr(%attr),
            content     => $attr{external} ? "" : $path_fileraw->slurp,
            signature   => $path_filesig->slurp,
        );
    my $status = $id ? $store->set(%args) : $store->add(%args);
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $status;

    # Remove temp files
    $path_fileraw->remove;
    $path_filesig->remove;

    # Result
    return $self->render(json => {
            status      => 1,
            message     => "Ok",
            number      => $rfcnum,
            size        => $self->param("size"),
            md5         => $self->param("md5"),
            published   => $self->param("published"),
            category    => $self->param("category"),
            title       => $self->param("title"),
            author      => $self->param("author"),
            attributes  => $self->param("attributes"),
        });
}
sub remove {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $store = $ctk->store;

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;

    # Delete record
    $store->del($rfcnum) or return $self->render(json => {
        status => 0,
        message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
    }, status => 500);

    # File in database directory
    my $filepath_raw = path($self->app->dbdir, $norfcnum->year,
        sprintf("%d%03d", $norfcnum->year, $norfcnum->yday),
        sprintf("%s.raw", $norfcnum->as_string))->remove; # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw>
    my $filepath_md5 = path($self->app->dbdir, $norfcnum->year,
        sprintf("%d%03d", $norfcnum->year, $norfcnum->yday),
        sprintf("%s.raw.md5", $norfcnum->as_string))->remove; # db/<Y3>/<Y3><YDAY>/<DOCNUM.raw.md5>

    # Result
    #$self->res->headers->content_type(CONTENT_TYPE_JSON);
    return $self->render(json => {
            status      => 1,
            message     => "Ok",
            number      => $rfcnum,
        });
}
sub reladd {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $rfcext = $self->param("external");
    my $name = $self->param("name");
    my $store = $ctk->store;

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Check external number
    my $norfcext = $ctk->is_valid($rfcext, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect external document number",
        }, status => 400) unless $norfcext;

    # Add relation
    my $id = $store->reladd(
            number  => $norfcnum->as_string,
            external=> $norfcext->as_string,
            name    => $name,
        );
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $id;

    # Result
    return $self->render(json => {
        status  => 1,
        message => "Ok",
        id      => $id,
        number  => $norfcnum->as_string,
        external=> $norfcext->as_string,
    });
}
sub relget {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $rfcext = $self->param("external");
    my $store = $ctk->store;

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Get by rfcext
    my $rfcext_number = "";
    if ($rfcext) {
        # Check external number
        my $norfcext = $ctk->is_valid($rfcext, QUIET_MODE);
        return $self->render(json => {
                status => 0,
                message => "Incorrect external document number",
            }, status => 400) unless $norfcext;
        $rfcext_number = $norfcext->as_string;
    }

    # Get relations
    my @relations = $store->relget($norfcnum->as_string, $rfcext_number);
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Result
    return $self->render(json => {
        status  => 1,
        message => scalar(@relations) ? "Found" : "Not found",
        relations => [@relations],
        number  => $norfcnum->as_string,
        external=> $rfcext_number,
    });
}
sub reldel {
    my $self = shift;
    my $ctk = $self->app->ctk();
    my $rfcnum = $self->param("rfcnum");
    my $rfcext = $self->param("external");
    my $store = $ctk->store;

    # Check number
    my $norfcnum = $ctk->is_valid($rfcnum, QUIET_MODE);
    return $self->render(json => {
            status => 0,
            message => "Incorrect document number",
        }, status => 400) unless $norfcnum;

    # Store check
    return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500) unless $store->status;

    # Delete by rfcext
    my $rfcext_number = "";
    if ($rfcext) {
        # Check external number
        my $norfcext = $ctk->is_valid($rfcext, QUIET_MODE);
        return $self->render(json => {
                status => 0,
                message => "Incorrect external document number",
            }, status => 400) unless $norfcext;
        $rfcext_number = $norfcext->as_string;
    }

    # Delete relation
    $store->reldel($norfcnum->as_string, $rfcext_number)
        or return $self->render(json => {
            status => 0,
            message => $store->error || sprintf("Store (%s): Unknown error", $store->dsn),
        }, status => 500);

    # Result
    return $self->render(json => {
        status  => 1,
        message => "Ok",
        number  => $norfcnum->as_string,
        external=> $rfcext_number,
    });
}

1;

__END__
